#!ruby -Ks

require 'uconv'

require 'grid.rb'


class SimpleFont
	def initialize
		@name = "lr SVbN"
		@size = 14
		@color = 0
		@style = 0
	end
	
	attr_accessor :name
	attr_accessor :size
	attr_accessor :color
	attr_accessor :style
	
	def save(io)
		n = Uconv.sjistou8(@name)
		Marshal.dump(n,io)
		Marshal.dump(@size,io)
		Marshal.dump(@color,io)
		Marshal.dump(@style,io)
	end
	
	def load(io)
		n = Marshal.load(io)
		@name = Uconv.u8tosjis(n)
		@size = Marshal.load(io)
		@color = Marshal.load(io)
		@style = Marshal.load(io)
	end
end



#̃J[hortH_ǂ̈ʒuɂǂ̗lȑ傫ŁA
#ǂ̗lȃIuWFNg(X^C)Ă邩𖾊mɂ
class Style
	def initialize
		@x = 16	#ẅʒu
		@y = 16	#ẍʒu
		@w = 16	#
		@h = 16	#c
		@shape = :fillRect	#J[hortH_̌`(lpAۂȂǂ̌`)
		
	end
	
	attr_accessor :x
	attr_accessor :y
	attr_accessor :w
	attr_accessor :h
	attr_accessor :shape
	
	def argument
		[@x , @y , @x+@w , @y+@h]
	end
	
	def save(io)
		Marshal.dump(@shape, io)
		Marshal.dump(@x, io)
		Marshal.dump(@y, io)
		Marshal.dump(@w, io)
		Marshal.dump(@h, io)
	end
	
	def load(io)
#		@shape = Marshal.load(io)
		@x = Marshal.load(io)
		@y = Marshal.load(io)
		@w = Marshal.load(io)
		@h = Marshal.load(io)
	end
end

class CardStyle < Style
	def initialize(txt="")
		super()
		@text = txt
		
		@font = SimpleFont.new
		
		@text_x_position = 1	#0:LEFT 1:CENTER 2:RIGHT
		@text_y_position = 0	#0:TOP 1:CENTER 2:BOTTOM
		
	end
	
	def text_x
		return @x+1
	end
	
	def text_y
		return @y+1
	end
	
	def text_w
		return @w-2
	end
	
	def text_h
		return @h-2
	end
	
	
	def save(io)
		super
		t = Uconv.sjistou8(@text)
		Marshal.dump(t, io)
		Marshal.dump(@text_x_position, io)
		Marshal.dump(@text_y_position, io)
		@font.save(io)
	end
	
	def load(io)
		super
		t = Marshal.load(io)
		@text = Uconv.u8tosjis(t)
		@text_x_position = Marshal.load(io)
		@text_y_position = Marshal.load(io)
		@font.load(io)
	end
	
	attr_accessor :text
	
	attr_accessor :font
	attr_accessor :font_name
	attr_accessor :font_size
	attr_accessor :font_color
	
	attr_accessor :text_x_position
	attr_accessor :text_y_position
	
	def font_argument
		[
			@text ,text_x ,text_y ,text_x + text_w ,text_y + text_h ,
			@text.size,@text_x_position
		]
	end
end

class CardStyle_fillRect < CardStyle
end

class CardStyle_fillRoundRect < CardStyle
	def initialize
		super
		@rx = 0
		@ry = 0
		
		@shape = :fillRoundRect
	end
	
	def argument
		[@x , @y , @x+@w , @y+@h , @rx , @ry]
	end
	attr_accessor :rx
	attr_accessor :ry
	
	def save(io)
		super
		Marshal.dump(@rx, io)
		Marshal.dump(@ry, io)
	end
	
	def load(io)
		super
		@rx = Marshal.load(io)
		@ry = Marshal.load(io)
	end

end

class CardStyle_fillEllipse < CardStyle
	def initialize
		super
		
		@shape = :fillEllipse
	end
end

class CardStyle_fillArc < CardStyle
	def initialize
		super
		@lxs = 0
		@lys = 0
		@lxe = 0
		@lye = 0
		
		@shape = :fillArc
	end
	
	def argument
		[@x , @y , @x+@w , @y+@h , @lxs , @lys , @lxe , @lye]
	end
	attr_accessor :lxs
	attr_accessor :lys
	attr_accessor :lxe
	attr_accessor :lye
	
	def save(io)
		super
		Marshal.dump(@lxs, io)
		Marshal.dump(@lys, io)
		Marshal.dump(@lxe, io)
		Marshal.dump(@lye, io)
	end
	
	def load(io)
		super
		@lxs = Marshal.load(io)
		@lys = Marshal.load(io)
		@lxe = Marshal.load(io)
		@lye = Marshal.load(io)
	end
	
end

class CardStyle_drawLine < CardStyle
	def initialize
		super
		
		@shape = :drawLine
	end
end

class CardStyle_drawChord < CardStyle
	def initialize
		super
		@lxs = 0
		@lys = 0
		@lxe = 0
		@lye = 0
		
		@shape = :drawChord
	end
	
	def argument
		[@x , @y , @x+@w , @y+@h , @lxs , @lys , @lxe , @lye]
	end
	attr_accessor :lxs
	attr_accessor :lys
	attr_accessor :lxe
	attr_accessor :lye
	
	def save(io)
		super
		Marshal.dump(@lxs, io)
		Marshal.dump(@lys, io)
		Marshal.dump(@lxe, io)
		Marshal.dump(@lye, io)
	end
	
	def load(io)
		super
		@lxs = Marshal.load(io)
		@lys = Marshal.load(io)
		@lxe = Marshal.load(io)
		@lye = Marshal.load(io)
	end
	
end

=begin
shape
EfillRect		(xstart,ystart,xend,yend) 
EfillRoundRect	(xstart,ystart,xend,yend,rx,ry) 
EfillEllipse	(xstart,ystart,xend,yend) 
EfillArc		(xstart,ystart,xend,yend,lxs,lys,lxe,lye) 
EdrawLine		(xstart,ystart,xend,yend) 
EdrawChord		(xstart,ystart,xend,yend,lxs,lys,lxe,lye) 

=end



class Item				#J[htH_
	def initialize(k,txt,c = :fillRect,p=nil)
		@name = txt
		@parent = nil
		@text = txt
		
		@kind = k	#:item :card :folder
		
		@shape = c
		
		@style = card_shape.new(@text)
		
		@note = ""
		
		@items = []
		
		
		if (@kind == :item)or(@kind == :card) then
			@sheetView_visible = true
		else
			@sheetView_visible = false
		end
		
		if (@kind == :item)or(@kind == :folder) then
			@shelfTreeview_visible = true
		else
			@shelfTreeview_visible = false
		end
		
		@grid = Grid.new
		
	end
	
	def card_shape
		case @shape
		when :fillRect
#			CardStyle_fillRect.new(@text)
			CardStyle_fillRect
#			send("CardStyle_#{@shape}")
		end
	end
	
	def save(io)
		n = Uconv.sjistou8(@name)
		Marshal.dump(n, io)
		t = Uconv.sjistou8(@text)
		Marshal.dump(t, io)
		Marshal.dump(@kind, io)
		Marshal.dump(@shape, io)
		nt = Uconv.sjistou8(@note)
		Marshal.dump(nt, io)
		
		@grid.save(io)
		
		@style.save(io)
		Marshal.dump(@items.size, io)
		@items.each{|e|
			e.save(io)
		}
	end
	
	def load(io)
		n = Marshal.load(io)
		@name = Uconv.u8tosjis(n)
		t = Marshal.load(io)
		@text = Uconv.u8tosjis(t)
		@kind = Marshal.load(io)
		@shape = Marshal.load(io)
		nt = Marshal.load(io)
		@note = Uconv.u8tosjis(nt)
		
		@grid.load(io)
		
		s =Marshal.load(io)
		case s
		when :fillRect
			@style = CardStyle_fillRect.new(@text)
		end
		@style.shape = s
		@style.load(io)
		@items =[]
		i = Marshal.load(io)
		if (0 < i) then
			0.upto(i-1){|k|
				@items[k] = Item.new(:item,"")
				@items[k].load(io)
				@items[k].parent = self
			}
		end

	end
	
	attr_accessor :name
	attr_accessor :parent
#	attr_accessor :text
	
	attr_accessor :kind
	
	attr_accessor :shape
	
	attr_accessor :font_name
	attr_accessor :font_size
	attr_accessor :font_color
	
	attr_accessor :font
	
	attr_accessor :style
	
	attr_accessor :note
	
	attr_accessor :items
	
	attr_accessor :sheetView_visible
	attr_accessor :shelfTreeview_visible
	
	attr_accessor :grid
	
	def text=(other)
		@text = other
		@style.text = other
	end
	
	def text
		@text
	end
	
	#insert
	def insert(i)
		i.parent = self
		self.items.each{|c|
			if (c.style.x <= i.style.x)and(i.style.x <= (c.style.x+c.style.w)) then
				if(c.style.y <= i.style.y)and(i.style.y <= (c.style.y+c.style.h)) then
					i.style.x = c.style.x + c.style.y
					i.style.y = c.style.y + c.style.h
				end
			end
		}
		self.items << i
	end
	
	#delete
	def delete(i)
		self.items.delete(i)
	end
	
	#R\[\֐(eXgp)
	def print_folder(f = self,i = 1)	#fŎw肵tH_ȉ̃tH_\
		s = "."
		
		print s*i + "#{f.text}\t\t"
		if f.parent != nil then
			print "(parent : " + "#{f.parent.text}" + ")\n"
		else
			print "(parent : self)\n"
		end
		if f.folders.empty? == false then
			f.folders.each_index do |index|
				i += 1
				temp_f = f.folders[index]
				
				print_folder(temp_f,i)
				
				i -= 1
			end
		end
		
	end
=begin
	def add(f)
		f.parent = self.parent
		
		if (self == self.parent.folders.last) then
			self.parent.folders << f
		else
			i = self.parent.folders.index(self)
			self.parent.folders = 
				self.parent.folders[0..i] .push(f) + 
				self.parent.folders[i+1..self.parent.folders.size]
		end
	end
=end
end


class Mark
	def initialize(c)
		@card = c
		@mark = @card.style
		@marks = []
		
		@marks[0] = Style.new
		@marks[0].shape = :fillRect
		@marks[0].x = @mark.x-2
		@marks[0].y = @mark.y-2
		@marks[0].w = 4
		@marks[0].h = 4
		
		@marks[1] = Style.new
		@marks[1].shape = :fillRect
		@marks[1].x = @mark.x-2
		@marks[1].y = @mark.y+@mark.h-2
		@marks[1].w = 4
		@marks[1].h = 4
		
		@marks[2] = Style.new
		@marks[2].shape = :fillRect
		@marks[2].x = @mark.x+@mark.w-2
		@marks[2].y = @mark.y+@mark.h-2
		@marks[2].w = 4
		@marks[2].h = 4
		
		@marks[3] = Style.new
		@marks[3].shape = :fillRect
		@marks[3].x = @mark.x+@mark.w-2
		@marks[3].y = @mark.y-2
		@marks[3].w = 4
		@marks[3].h = 4
		
		if (8 < @mark.w) then
			@marks[4] = Style.new
			@marks[4].shape = :fillRect
			@marks[4].x = @mark.x+(@mark.w / 2)-2
			@marks[4].y = @mark.y-2
			@marks[4].w = 4
			@marks[4].h = 4
			
			@marks[5] = Style.new
			@marks[5].shape = :fillRect
			@marks[5].x = @mark.x+(@mark.w / 2)-2
			@marks[5].y = @mark.y+@mark.h-2
			@marks[5].w = 4
			@marks[5].h = 4
			
		else
			@marks[4] = nil
			@marks[5] = nil
		end
		
		if(8 < @mark.h) then
			@marks[6] = Style.new
			@marks[6].shape = :fillRect
			@marks[6].x = @mark.x-2
			@marks[6].y = @mark.y+(@mark.h / 2)-2
			@marks[6].w = 4
			@marks[6].h = 4
			
			@marks[7] = Style.new
			@marks[7].shape = :fillRect
			@marks[7].x = @mark.x+@mark.w-2
			@marks[7].y = @mark.y+(@mark.h / 2)-2
			@marks[7].w = 4
			@marks[7].h = 4

		else
			@marks[6] = nil
			@marks[7] = nil
		end
	end
	
	attr_accessor :marks
end

